//+------------------------------------------------------------------+
//|                                              stochastic momentum |
//|                                                           mladen |
//|                                                                  |
//| Stochastic momentum developed by William Blau                    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    2
#property indicator_color1     White
#property indicator_color2     Red
#property indicator_levelcolor DarkSlateGray


//
//
//
//
//

extern int    Length           = 14;
extern int    Smooth1          = 10;
extern int    Smooth2          = 14;
extern int    Signal           = 5;
extern int    Price            = PRICE_CLOSE;
extern double upperLevel       = 40.0;
extern double lowerLevel       = -40.0;

extern string note             = "turn on Alert = true; turn off = false";
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = true;
extern bool   alertsNotify     = true;
extern bool   alertsEmail      = false;
extern string soundFile        = "alert2.wav";


//
//
//
//
//

double smo[];
double sms[];
double trend[];
double tBuffer[][6];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,smo); SetIndexLabel(0,"Stochastic momentum");
   SetIndexBuffer(1,sms); SetIndexLabel(1,"Stochastic momentum signal");
   SetIndexBuffer(2,trend);
   SetLevelValue(0,upperLevel);
   SetLevelValue(1,0);
   SetLevelValue(2,lowerLevel);

   //
   //
   //
   //
   //

   Length  = MathMax(Length ,1);
   Smooth1 = MathMax(Smooth1,1);
   Smooth2 = MathMax(Smooth2,1);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName (" Stochastic momentum ("+Length+","+Smooth1+","+Smooth2+","+Price+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define ema10 0
#define ema11 1
#define ema12 2
#define ema20 3
#define ema21 4
#define ema22 5

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
                 if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);

         double alpha1 = 2.0 /(1.0+Smooth1);
         double alpha2 = 2.0 /(1.0+Smooth2);
         double alphas = 2.0 /(1.0+Signal);
         
   //
   //
   //
   //
   //
   //

   for(i=limit,r=Bars-i-1; i >= 0; i--,r++)
   {
      double hh = High[iHighest(NULL,0,MODE_HIGH,Length,i)];
      double ll =  Low[ iLowest(NULL,0,MODE_LOW ,Length,i)];
      double pr = iMA(NULL,0,1,0,MODE_SMA,Price,i);

         tBuffer[r][ema10] = pr - 0.5*(hh+ll);
         tBuffer[r][ema20] =           hh-ll;
         if (i>=Bars-3)
         {
            tBuffer[r][ema11] = tBuffer[r][ema10];
            tBuffer[r][ema12] = tBuffer[r][ema10];
            tBuffer[r][ema21] = tBuffer[r][ema20];
            tBuffer[r][ema22] = tBuffer[r][ema20];
            continue;
         }
      
      //
      //
      //
      //
      //
      
      tBuffer[r][ema11] = tBuffer[r-1][ema11] + alpha1*(tBuffer[r][ema10]-tBuffer[r-1][ema11]);
      tBuffer[r][ema12] = tBuffer[r-1][ema12] + alpha2*(tBuffer[r][ema11]-tBuffer[r-1][ema12]);
      tBuffer[r][ema21] = tBuffer[r-1][ema21] + alpha1*(tBuffer[r][ema20]-tBuffer[r-1][ema21]);
      tBuffer[r][ema22] = tBuffer[r-1][ema22] + alpha2*(tBuffer[r][ema21]-tBuffer[r-1][ema22]);

         smo[i] = 100.00 * tBuffer[r][ema12] / (0.5 * tBuffer[r][ema22]);
         if (Signal>1)
            sms[i]   = sms[i+1]+alphas*(smo[i]-sms[i+1]);
            trend[i] = trend[i+1];
            if (smo[i]>upperLevel) trend[i] =-1;
            if (smo[i]<lowerLevel) trend[i] = 1;
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("uptrend");
      else  doAlert("downtrend");       
   }
 return(0);
}

//+------------------------------------------------------------------+


void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," SMI ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," SMI " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," SMI "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}
